import os
import requests
import time

from retry import retry
from synativ.client import SynativApi
from typing import List


def upload_dataset(api: SynativApi, dataset_id: str, dataset_dir: str):
    dataset_files = _list_files(dataset_dir=dataset_dir)
    for filename in dataset_files:
        print(f"Uploading {filename} to {dataset_id}")
        presigned_url: str = api.get_upload_url(dataset_id=dataset_id, filename=filename).presigned_url
        filepath = os.path.join(dataset_dir, filename)
        _upload_file(presigned_url=presigned_url, local_filepath=filepath)
        time.sleep(2)


def _list_files(dataset_dir: str) -> List[str]:
    filenames = []
    for filename in os.listdir(dataset_dir):
        filepath = os.path.join(dataset_dir, filename)
        if os.path.isfile(filepath):
            filenames.append(filename)
        else:
            print(f"Ignoring subdirectory: {filename}")
    return filenames


@retry(tries=5, delay=2, backoff=2)
def _upload_file(presigned_url: str, local_filepath: str):
    with open(local_filepath, "rb") as f:
        data = f.read()
    response = requests.put(url=presigned_url, data=data)
    print(response.status_code)
